from Products.azax.plugins.core.interfaces import IKSSCoreCommands
from Products.azax import force_unicode
from Products.CMFCore.utils import getToolByName
from DocumentTemplate import sequence as Sequence
from Products.BatchIt.config import SORT_STRINGS
from Products.CMFPlone import Batch
from zope.component import getMultiAdapter
from Acquisition import aq_inner
from Products.BatchIt.browser.batchview import BatchView

class TableView(BatchView):
    """ View class to enable kss-based batch sorting on folder listings
    """
    
    def getFilteredContent(self, contentFilter = None, batch = False, b_size = 100, full_objects = False):
        """ Get filtered content objects and store their paths in a session """

        context = aq_inner(self.context)
        contents = context.getFolderContents(contentFilter, batch, b_size, full_objects)

        if batch:
            b = {}
            b['start'] = contents.start - 1
            b['size'] = contents.size
            b['sequence'] = [c.getPath() for c in contents._sequence]
            session = self.request.SESSION
            session['batch'] = b

        return contents
    
    def manipulateBatch(self, field, sort):
        """ Perform the column sort and rerender the page """
        
        sort_id = sort
        sort_query = sort_id
        if sort_query == 'none':
            sort_query = 'asc'
        elif sort_query == 'asc':
            sort_query = 'desc'
        elif sort_query == 'desc':
            sort_query = 'asc'
        self._reorderArrows(field, sort_id, sort_query)
        
        session = self.request.SESSION
        if session.has_key('batch'):
            b = session['batch']
            b_sort = (SORT_STRINGS[field], 'cmp', sort_query)
            self._constructTable(b['start'], b['size'], b['sequence'], b_sort)
    
        return self.render()
    
    def _constructTable(self, start, size, sequence, sort = None):
        """ Helper method to recreate the sorted table """
        
        context = aq_inner(self.context)
        putils = getToolByName(context, 'plone_utils')
        mtool = getToolByName(context, 'portal_membership')
        proptool = getToolByName(context, 'portal_properties')
        portal_catalog = getToolByName(context, 'portal_catalog')
        ploneview = getMultiAdapter((context, self.request), name = 'plone')
        
        seq = []
        path = {}
        path['depth'] = 0
        contentFilter = {}
        for p in sequence:
            path['query'] = p
            contentFilter['path'] = path
            seq += portal_catalog.queryCatalog(contentFilter)
            
        if sort:
            seq = Sequence.sort(seq, (sort, ))
        
        session = self.request.SESSION
        if session.has_key('batch'):
            b = session['batch']
            b['start'] = start
            b['sequence'] = [s.getPath() for s in seq]
            session['batch'] = b
        
        batch = Batch(seq, size, start)
        
        show_about = not mtool.isAnonymousUser() or proptool.site_properties.allowAnonymousViewAbout
        html = ""
        
        for n in range(batch.first, batch.end):
            s = batch._sequence[n]
            item_number = str(n+1)
            item_type = s.portal_type
            url = s.getURL() + '/'
            if item_type in proptool.site_properties.typesUseViewActionInListings:
                url += 'view'
            item_creator = s.Creator
            author = mtool.getMemberInfo(item_creator)
                
            html += """<tr class="%s"><td id="title-%s"><span class="visualIcon contenttype-%s">""" \
                         % (n%2 == 0 and 'odd' or 'even',
                            item_number,
                            putils.normalizeString(item_type))
            html += """<a href="%s" class="state-%s visualIconPadding" title="%s"> %s </a></span></td>""" \
                         % (url,
                            putils.normalizeString(s.review_state),
                            s.Description,
                            s.pretty_title_or_id())
            html += """<td id="author-%s"><a href="%s/author/%s">%s</td>""" \
                         % (item_number,
                            context.portal_url(),
                            item_creator,
                            author and author['fullname'] or item_creator)
            html += """<td id="type-%s">%s</td>""" \
                         % (item_number,
                            s.Type)
            html += """<td id="modified-%s">%s</td></tr>""" \
                         % (item_number,
                            ploneview.toLocalizedTime(s.ModificationDate, long_format = 1))
        
        html = force_unicode(html, 'utf')
        IKSSCoreCommands(self).replaceInnerHTML(
            IKSSCoreCommands(self).getHtmlIdSelector('batchtable'), html)
    
