from Products.azax.plugins.core.interfaces import IKSSCoreCommands
from Products.azax import force_unicode
from Products.CMFCore.utils import getToolByName
from DocumentTemplate import sequence as Sequence
from Products.CMFPlone import Batch
from Acquisition import aq_inner
from Products.BatchIt.browser.batchview import BatchView

class TopicView(BatchView):
    """ View class to enable kss-based batch sorting on smart folder listings
    """
    
    def getFilteredContent(self, contentFilter = None, batch = False, b_size = 100, full_objects = False):
        """ Get filtered content objects and store their paths in a session """

        context = aq_inner(self.context)
        contents = context.queryCatalog(None, batch, b_size, full_objects)

        if batch:
            b = {}
            b['start'] = contents.start - 1
            b['size'] = contents.size
            b['sequence'] = [c.getPath() for c in contents._sequence]
            session = self.request.SESSION
            session['batch'] = b

        return contents

    def manipulateBatch(self, field, sort):
        """ Perform the column sort and rerender the page """
        
        context = aq_inner(self.context)
        sort_id = sort
        sort_query = sort_id
        if sort_query == 'none':
            sort_query = 'asc'
        elif sort_query == 'asc':
            sort_query = 'desc'
        elif sort_query == 'desc':
            sort_query = 'asc'
        self._reorderArrows(field, sort_id, sort_query)
        
        session = self.request.SESSION
        if session.has_key('batch'):
            b = session['batch']
            vocab = context.listMetaDataFields(False)
            b_sort = (vocab.getKey(field, field), 'cmp', sort_query)
            self._constructTable(b['start'], b['size'], b['sequence'], b_sort)
    
        return self.render()
    
    def _constructTable(self, start, size, sequence, sort = None):
        """ Helper method to recreate the sorted table """
        
        context = aq_inner(self.context)
        putils = getToolByName(context, 'plone_utils')
        proptool = getToolByName(context, 'portal_properties')
        portal_catalog = getToolByName(context, 'portal_catalog')
        
        seq = []
        path = {}
        path['depth'] = 0
        contentFilter = {}
        for p in sequence:
            path['query'] = p
            contentFilter['path'] = path
            seq += portal_catalog.queryCatalog(contentFilter)
        
        if sort:
            seq = Sequence.sort(seq, (sort, ))
        
        session = self.request.SESSION
        if session.has_key('batch'):
            b = session['batch']
            b['start'] = start
            b['sequence'] = [s.getPath() for s in seq]
            session['batch'] = b
            
        batch = Batch(seq, size, start)
        
        fields = context.getCustomViewFields()
        html = ""
        
        for n in range(batch.first, batch.end):
            s = batch._sequence[n]
            item_number = str(n+1)
            item_type = s.portal_type
            item_url = s.getURL() + '/'
            if item_type in proptool.site_properties.typesUseViewActionInListings:
                item_url += 'view'
            item_icon = context.portal_url() + '/' + context.portal_types.getTypeInfo(item_type).getIcon()
            html += """<tr class="%s">""" % (n%2 == 0 and 'odd' or 'even')
            for f in fields:
                if f == 'Title': 
                    html += """<td id="title-%s" style="white-space: nowrap"><a href="%s"><img src="%s" height="16" width="16"/></a>""" \
                                 % (item_number,
                                    item_url,
                                    item_icon)
                    if item_type not in ['Link', 'Image']:
                        html += """<a href="%s/view" class="state-%s">%s</a>""" \
                                 % (item_url,
                                    putils.normalizeString(s.review_state),
                                    s.pretty_title_or_id())
                    if item_type == 'Image':
                        html += """<a href="%s" class="state-%s">%s</a>""" \
                                 % (item_url,
                                    putils.normalizeString(s.review_state),
                                    s.pretty_title_or_id())
                    if item_type == 'Link':
                        html += """<a href="%s" class="state-%s link-plain">%s</a>""" \
                                 % (s.getRemoteUrl(),
                                    putils.normalizeString(s.review_state),
                                    s.pretty_title_or_id())
                    html += """</td>"""
                else:
                    value = getattr(s, f, None)
                    html += """<td id="%s-%s">%s</td>""" \
                                 % (f.lower(),
                                    item_number,
                                    context.formatCatalogMetadata(value))
            html += """</tr>"""
        
        html = force_unicode(html, 'utf')
        IKSSCoreCommands(self).replaceInnerHTML(
            IKSSCoreCommands(self).getHtmlIdSelector('batchtable'), html)
